# -*- coding: utf-8 -*-

"""
upraise.util
~~~~~~~~~~~~

Utility code.

:date: $Date: 2007-07-20 23:31:15 +0200 (Fr, 20 Jul 2007) $
:revision: $Rev: 165 $
:copyright: 2006-2007 Jochen Kupperschmidt
:license: MIT, see LICENSE for details.
"""

from threading import local

from paste.httpexceptions import get_exception, HTTPFound
from paste.util.import_string import try_import_module
from paste.script.templates import Template
from paste.wsgiwrappers import WSGIRequest


__all__ = ['config', 'request', 'Request', 'get_action', 'call_action',
    'redirect_to', 'UpraiseTemplate']


# ---------------------------------------------------------------- #
# thread-locals

thread_local = local()


class Request(WSGIRequest):

    def __init__(self, environ):
        WSGIRequest.__init__(self, environ)
        thread_local.request = self


class ThreadedRequest(object):
    """A thread-local proxy for a ``Request`` object."""

    def __getattr__(self, name):
        # Make ``dir()`` work on instances of this class.
        if name == '__members__':
            return [x for x in dir(thread_local.request)
                if not x.startswith('_')]

        return getattr(thread_local.request, name)

    def __setattr__(self, name, value):
        setattr(thread_local.request, name, value)


request = ThreadedRequest()


class Config(local):
    """A class to hold configuration data.

    Including functions specified by the application.
    """


config = Config()


# ---------------------------------------------------------------- #
# request exits

def abort(code, detail=None, headers=None, comment=None):
    """Immediately abort the request by returning an HTTP exception.

    As seen in Pylons (pylons.helpers.abort).
    """
    raise get_exception(code)(detail, headers, comment)

def redirect_to(*args, **kwargs):
    """Accept a WSGIResponse object and its headers before redirecting."""
    if not hasattr(config, 'url_for_func'):
        raise NotImplementedError(
            "No callable 'url_for_func' for redirecting was set.")
    response = kwargs.pop('_response', None)
    found = HTTPFound(config.url_for_func(*args, **kwargs))
    if response:
        found.headers.extend(response.wsgi_response()[1])
    raise found


# ---------------------------------------------------------------- #
# framework structure

def get_action(controller, action, package, classname='Controller', debug=False):
    """Instanciate the controller and return the action method."""
    mod_name = '%s.controllers.%s' % (package, controller)
    mod = try_import_module(mod_name)
    if mod is None:
        abort(404, debug and 'Controller module not found.' or None)
    try:
        ctrl_class = getattr(mod, classname)
    except AttributeError:
        abort(404, debug and 'Controller class not found.' or None)
    try:
        return getattr(ctrl_class(), action)
    except AttributeError:
        abort(404, debug and 'Controller action not found.' or None)

def call_action(environ, request, package):
    """Call an action according to routing args."""
    # Use results from routing middleware.
    match = environ['wsgiorg.routing_args'][1].copy()
    try:
        controller = match.pop('controller')
        action = match.pop('action')
    except KeyError:
        abort(404, 'Essential routing args missing.')

    # Assemble parameters.
    params = dict(request.params.mixed())
    # Remove id if None.
    if 'id' in match and match['id'] is None:
        del match['id']
    params.update(match)

    # Locate and call action method, passing request parameters.
    action = get_action(controller, action, package)
    resp = action(**params)
    if not hasattr(resp, '__call__'):
        abort(500, 'Controller action did not return a callable.')
    return resp


# ---------------------------------------------------------------- #
# paster stuff

class UpraiseTemplate(Template):
    """This (and the entry point in ``setup.py``) will allow to make use of
    the ``paster create --template=upraise`` command to create an example
    project skeleton.
    """

    _template_dir = 'templates/default_project'
    egg_plugins = ['Upraise']
    summary = 'Upraise application template'
